%% Lichun on 4/13/2017
%Fluid flow through sine wave geometry
%Figure 1: arrows indicates normalized flow velocity
%blue area represents immobile zone, red area represents main flow channel
%%
    clc;clear all;close all;
% data includes three columns : x y u
    load('Velocity_field.mat');
%% number of data in the x and y directions
    N=500; % in the x direction number of columns
    M=300; % in the y direction number of rows
    %%
    xx=reshape(data(:,1),N,[]);
    yy=reshape(data(:,2),N,[]);
    uu=reshape(data(:,3),N,[]);
    vv=reshape(data(:,4),N,[]);
    xx=xx';
    yy=yy';
    uu=uu';
    vv=vv';
    U_Mag=sqrt(uu.^2+vv.^2);
    dx=xx(1,2)-xx(1,1);
    dy=yy(2,1)-yy(1,1);
    %%
    figure(1);
%     h1=surf(xx,yy,uu);
%     set(h1,'edgecolor','none');
    quiver(xx(1:10:end,1:10:end),yy(1:10:end,1:10:end)...
        ,uu(1:10:end,1:10:end)./U_Mag(1:10:end,1:10:end)...
        ,vv(1:10:end,1:10:end)./U_Mag(1:10:end,1:10:end));
    view(2)
    axis equal;
    hold on
    %%
    z_top_surf=zeros(N,1);
    z_bot_surf=zeros(N,1);
    z_top_limit=zeros(N,1);%number of cells counting from top surface
    z_bot_limit=zeros(N,1);%number of cells counting from bottom surface
    uu_plot=uu;
    uu_plot(find(isnan(uu_plot)==0))=1;
    for j=1:N %loop over column
        value_old=0;
        value_current=0;
        for i=1:M % for top surfaces loop over rows
            if(isnan(uu(i,j))==0) % valid cell
                if(z_top_surf(j)==0) % find the first valid cell at the top surface
                    z_top_surf(j)=i;
                end
                if(sum(uu(:,j)<0)>=1) % if negative velocity was found near wall
                    if(uu(z_top_surf(j)+1,j)*uu(z_top_surf(j),j)<0)
                        uu(z_top_surf(j),j)=-uu(z_top_surf(j),j);
                    end
                    
                    value_old=value_current;
                    value_current=value_old+uu(i,j);                   
                    if(value_current*value_old<0)                        
                        if(value_current<0)
                            value_current=0;
                        elseif(value_current>0)
                            dh=abs(value_old/(-value_current+value_old));
                            z_top_limit(j)=i-z_top_surf(j)-dh;
                            break;
                        end
                    end 
                    
                end
            end        
        end
        if(z_top_limit(j)>0)
            uu_plot(z_top_surf(j):z_top_surf(j)+floor(z_top_limit(j)),j)=0;
        end    


        value_old=0;
        value_current=0;
        for i=M:-1:1 % for bottom surfaces  loop over rows
            if(isnan(uu(i,j))==0) % valid cell
                if(z_bot_surf(j)==0) % find the first valid cell at the bottom surface
                    z_bot_surf(j)=i;
                end
                if(sum(uu(:,j)<0)>=1) % if negative velocity was found near wall
                    
                    if(uu(z_bot_surf(j)-1,j)*uu(z_bot_surf(j),j)<0)
                        uu(z_bot_surf(j),j)=-uu(z_bot_surf(j),j);
                    end
                    
                    value_old=value_current;
                    value_current=value_old+uu(i,j);               
                    if(value_current*value_old<0)                        
                        if(value_current<0)
                            value_current=0;
                        elseif(value_current>0)
                            dh=abs(value_old/(-value_current+value_old));
                            z_bot_limit(j)=z_bot_surf(j)-i-dh;
                            break;
                        end
                    end                    
                end
            end        
        end
        if(z_bot_limit(j)>0)
            uu_plot(ceil(z_bot_surf(j)-z_bot_limit(j)):z_bot_surf(j),j)=0;
        end
    end
    %%
    z_top_limit(find(z_top_limit)<1)=0;
    z_bot_limit(find(z_bot_limit)<1)=0;
    Volume=sum(reshape((z_top_limit+z_bot_limit)*dx*dy,1,[]));
    %%
%     figure(2);
    title('Immobile area (in blue)','fontsize',20)
    h1=surf(xx,yy,uu_plot);
    set(h1,'edgecolor','none');
    alpha(0.5);
    view(2)
    set(gca,'fontsize',20)
    axis equal;